// ########################################################################
// ## mops.cc                                                            ##
// ## Zufllige Erzeugung von MOP-Graphen                                ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#define MopsName "Mops 1.0"

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

#include "../tdgf/tdgf.h"
#include "../tdgf/graph.h"
#include "../tdgf/graph.icc"
#include "../zufall/zufall.h"


// Zufllige Erzeugung von MOPs mit "knoten" Knoten
// graph: Zeiger auf Graph mit "knoten" Knoten
// ring:  Array mit Eintrgen 1,...,knoten Knotenreihenfolge auf Ring
void MOP(Graph* graph, card32* ring, card32 knoten)
{
   if(knoten == 3) {
      // 3er-MOP erzeugen       
      ring[1] = 1;
      ring[2] = 2;
      ring[3] = 3;
      graph->Verbinde(0,1);
      graph->Verbinde(1,2);
      graph->Verbinde(2,0);      
   }
   else if(knoten < 3) {
      // Keine MOPs mit weniger als 3 Knoten!
      cerr << "MOP() - Es gibt keine MOPs mit weniger als 3 Knoten!" << endl;
      exit(1);
   }
   else {
      // MOP mit (knoten - 1) Knoten erzeugen
      MOP(graph,ring,knoten - 1);

      // Neuen Knoten hinzufgen: Auswahl einer Kante
      card32 x = Random(knoten - 1);

      // Neuen Knoten in den Ring einfgen     
      for(card32 i = knoten;i > x + 1;i--)
         ring[i] = ring[i - 1];
      ring[x + 1] = knoten;

      // Nachbarknoten bestimmen
      int a = ring[x];
      int b = ((x + 2) <= knoten) ? ring[x + 2] : ring[1];

      // Neuen Knoten mit Graphen verbinden
      graph->Verbinde(a - 1,knoten - 1);
      graph->Verbinde(knoten - 1,b - 1);
   }
}



// Hauptprogramm
int32 main(card32 argc, char** argv)
{
   card32 knoten    = 6;
   card32 anzahl    = 1;
   bool  bitMatrix  = true;
   char  titel[128];
   char* copyright  = NULL;
   char* programm   = MopsName;
   char* name       = NULL;
   bool  ausgabe    = false;
   ofstream           os;
   card32             i;

   if(argc == 1) {
      cerr << "Aufruf: " << argv[0] << " {Parameter}" << endl;
      cerr << "-k[#]    - Knotenzahl setzen" << endl;
      cerr << "-#[#]    - Anzahl der zu erzeugenden Graphen" << endl;
      cerr << "-f[Name] - Dateiname fr Ausgabe (TDGF)" << endl;
      cerr << "-t[Text] - Titel setzen" << endl;
      cerr << "-c[Text] - Copyright setzen" << endl;
      cerr << "-y       - Adjazenzmatrix im Byte-Format statt Bit-Format speichern" << endl;
      cerr << "-$       - Ausgabe zustzlich auf Bildschirm" << endl;
      exit(1);
   }

   // Parameter auswerten
   titel[0] = 0x00;
   for(i = 1;i < argc;i++) {
      if(!(strncmp(argv[i],"-t",2)))      strcpy(titel,&argv[i][2]);
      else if(!(strncmp(argv[i],"-c",2))) copyright = &argv[i][2];
      else if(!(strncmp(argv[i],"-k",2))) knoten    = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-#",2))) anzahl    = atoi(&argv[i][2]);
      else if(!(strncmp(argv[i],"-f",2))) name      = &argv[i][2];
      else if(!(strcmp(argv[i],"-y")))    bitMatrix = false;
      else if(!(strcmp(argv[i],"-$")))    ausgabe = true;  
   } 
   if(name == NULL) ausgabe = true;
   if(anzahl == 0) anzahl = 1;
   if(knoten < 3) knoten = 3;
   if(titel[0] == 0x00)
      sprintf(titel,"Zufllig erzeugte MOPs mit %d Knoten",knoten);
   if(name) {
      if(!ErstelleTDGF(os,name,titel,copyright,programm)) {
         cerr << "Kann Datei \"" << name << "\" nicht erstellen!" << endl;
         exit(1);
      }
   }

   // Zufallszahlengenerator initialisieren
   Randomize();

   // Informationen ausgeben
   cout << "Parameter fr die Generierung:" << endl;
   cout << "Knotenzahl: " << knoten << endl;
   cout << "Anzahl:     " << anzahl << endl;
   cout << "Titel:      " << titel << endl;
   cout << "Copyright:  " << copyright << endl << endl;
   

   // Graphen erzeugen
   for(card32 zaehler = 1;zaehler <= anzahl;zaehler++) {
      Graph  graph(knoten);
      card32 ring[knoten + 1];
      MOP(&graph,ring,knoten);
      
      Graph g(knoten);
      for(card32 a = 0;a < knoten;a++) {
         char label[12];
         sprintf(label,"%d",a + 1);
         g.SetzeLabel(a,label);

         for(card32 b = a + 1;b < knoten;b++) {
           if(graph.Verbunden(ring[a + 1] - 1,ring[b + 1] - 1)) {
              g.Verbinde(a,b);
            }
         }
      }
      if(name) g.Schreibe(os);

      if(ausgabe) {
         cout << "#" << zaehler << ":" << endl;
         cout << g << endl;
      }
   }
 
   return 0;
}
