// ########################################################################
// ## tdgf.h                                                             ##
// ## Prozeduren zum Bearbeiten von Dateien im TDGF-Format               ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#ifndef TDGF_H
#define TDGF_H

#include <iostream.h>
#include <fstream.h>

#include "../system.h"


// TDGF: Thomas Dreibholz Graph Format-Datensatz
#define TDGF_Header_ID               (char *)"TDGF"
#define TDGF_Header_ByteOrderTest    (card32)0x29091976
#define TDGF_Header_KeineKompression 0
#define TDGF_Header_Version          1
struct TDGF_Header
{
   char   ID[4];
   card32 Bytes;
   card32 ByteOrderTest;
   card16 Kompression;
   card16 Version;
};


// INFO: INFO-Datensatz
#define INFO_Header_StringLaenge (int)80
#define INFO_Header_ID (char *)"INFO"
struct INFO_Header
{
   char   ID[4]; 
   card32 Bytes;
   char   Titel[INFO_Header_StringLaenge];
   char   Copyright[INFO_Header_StringLaenge];
   char   Programm[INFO_Header_StringLaenge];
   card8  Tag,Monat;
   card16 Jahr;     
   card8  Stunde,Minute,Sekunde;
   char   Zone[4];
   char   null[5];
};

// GRDT: GRaph DaTen-Datensatz
#define GRDT_Header_ID (char *)"GRDT"
struct GRDT_Header
{
   char   ID[4];
   card32 Bytes;
   card16 Knoten;
   card16 Kanten;
   card16 Flags;
   card8  null[2];
   // Es folgt die Adjazenzmatrix in durch Flags gegebenen Format
};


// GRTT: GRaph Titel Text-Datensatz
#define GRTT_Header_ID (char *)"GRTT"
struct GRTT_Header
{
   char   ID[4];
   card32 Bytes;
   // Es folgt der Titel-String, mit 0-Byte abgeschlossen
};


// GRLD: GRaph Label Daten-Datensatz
#define GRLD_Header_ID (char *)"GRLD"
struct GRLD_Header
{
   char   ID[4];
   card32 Bytes;
   // Es folgt die Label-Tabelle mit 4 Zeichen pro Label
};


// GRBD: GRaph Bipart Daten-Datensatz
#define GRBD_Header_ID (char *)"GRBD"
struct GRBD_Header
{
   char   ID[4];
   card32 Bytes;
   // Es folgt die Biparts-Liste: Bitweise in Bytes gepackt
};


// Fehler von OeffneTDGF()
enum tdgfFehler { OK = 0, dos = 1, format = 2, byteOrder = 3, kompression = 4 };


// Funktionen zum Lesen und Schreiben von TDGF-Dateien
extern bool ErstelleTDGF(ofstream& os, const char* name,
                         const char* titel     = NULL,
                         const char* copyright = NULL,
                         const char* programm  = NULL);
                      
extern tdgfFehler OeffneTDGF(ifstream& is, const char* name,
                             TDGF_Header* tdgf, INFO_Header* info);


// Interne Funktionen - Nicht zur allgemeinen Benutzung
extern bool SchreibeTDGF(ostream& os);
extern bool SchreibeINFO(ostream& os, const char* titel, const char* copyright,
                                      const char* programm);                              
extern bool SchreibeGRDT(ostream& os, card16 flags,  card16 knoten, 
                                      card16 kanten, card16 adjazenzBytes);
extern bool SchreibeGRTT(ostream& os, const char* titel);
extern bool SchreibeGRLD(ostream& os, card16 knoten, char* labels);
extern bool SchreibeGRBD(ostream& os, card16 knoten, card8* biparts);

#endif
