// ########################################################################
// ## graphwidget.cc                                                     ##
// ## Viewer fr Graphenlisten                                           ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#include <iostream.h>
#include <fstream.h>
#include <math.h>

#include <qapp.h>
#include <qfont.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qfiledlg.h>
#include <qmenubar.h>
#include <qscrollbar.h>
#include <qpopmenu.h>
#include <qkeycode.h>
#include <qmsgbox.h>
#include <qprogdlg.h>

#include <kapp.h>
#include <ktopwidget.h>
#include <ktoolbar.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kprogress.h>

#include "../system.h"
#include "../tdgf/graph.h"
#include "../tdgf/graph.icc"

#include "graphwidget.h"
#include "graphwidget.icc"


// Konstruktor fr Graph-Ausgabe
GraphWidget::GraphWidget(QWidget* parent, const char* name)
 : QWidget(parent,name)
{
   setMinimumSize(200,80);
   anzahlX   = 5;
   anzahlY   = 4;
   position  = 0;   
   seite     = 1; 
   liste     = new QList<Graph>;
   scrollBar = new QScrollBar(1,1,1,5,1,QScrollBar::Vertical,this);
   scrollBar->setGeometry(width() - 20,0,20,height());
   connect(scrollBar,SIGNAL(valueChanged(int)),this,SLOT(scrollBarEvent(int)));
   NeueSeite();
}

     
// Desstruktor fr Graph-Ausgabe
GraphWidget::~GraphWidget()
{
   delete liste;
}


// paintEvent-Handler
void GraphWidget::paintEvent(QPaintEvent *)
{
   QPainter painter;
   painter.begin(this);
   painter.eraseRect(0,0,width(),height());
   if(liste->count() > 0) {
      ZeichneSeite(position,&painter,0,0,width() - 20,height());
   }
   else {
      painter.setFont(QFont("Times",18,QFont::Bold));
      painter.drawText(0,0,width(),height(),AlignHCenter | AlignVCenter,
                       i18n("Please load a graph file!"));
   }
   painter.end();
   scrollBar->setGeometry(width() - 20,0,20,height());
}


// Graph zeichnen
void GraphWidget::ZeichneGraph(QPainter* painter,
                               const card32 x,
                               const card32 y,
                               const card32 breite,
                               const card32 hoehe,
                               Graph* graph,
                               const card32 num) const
{
   painter->setPen(black);
   painter->drawRect(x,y,breite,hoehe);
   if(graph != NULL) {
      // Knoten zeichnen
      const card32 mx = x + (breite / 2);
      const card32 my = y + (hoehe / 2);
      const double hx = (double)breite / 2.8;
      const double hy = (double)hoehe  / 2.8;
      const card32  N = graph->Knoten();
      double gp       = 2.0 * M_PI / (double)N;
      card32 i,j;
      card32 cx[N],cy[N];
      char   str[16];

      // Knoten zeichnen
      painter->setPen(black);
      if(graph->Biparts() == NULL) {
         for(i = 0;i < N;i++) {
            cx[i] = (int32)mx + (int32)(hx * cos((double)i * gp - M_PI_2));
            cy[i] = (int32)my + (int32)(hy * sin((double)i * gp - M_PI_2));
            if((!graph->Label(i,str))) {
               painter->drawEllipse(cx[i] - 2,cy[i] - 2,4,4);
            }
         }
      }
      else {
         j = 0;
         for(i = 0;i < N;i++) {
            if(graph->Bipart(i) == true) j++;
         }
         if(j == 1) {
            cx[0] = (int32)(mx - hx);
            cy[0] = (int32)my;
         }
         else if(j > 1) {
            for(i = 0;i < j;i++) {
               cx[i] = (int32)(mx - hx);
               cy[i] = (int32)(my - hy + i * (2.0 * hy / (j - 1)));
            }
         }
         if(j == N - 1) {
            cx[j] = (int32)(mx + hx);
            cy[j] = (int32)my;
         }
         else if(j < N - 1) {
            for(i = j;i < N;i++) {
               cx[i] = (int32)(mx + hx);
               cy[i] = (int32)(my - hy + (i - j) * (2.0 * hy / (N - j - 1)));
            }            
         }
         for(i = 0;i < N;i++) {
            if((!graph->Label(i,str))) {
               painter->drawEllipse(cx[i] - 2,cy[i] - 2,4,4);
            }
         }
      }

      // Kanten zeichnen
      painter->setPen(black);
      for(i = 0;i < N;i++) {
         for(j = i + 1;j < N;j++) {
            if(graph->Verbunden(i,j)) {
               painter->drawLine(cx[i],cy[i],cx[j],cy[j]);  
            }
         }
      }

      // Label ausgeben
      if(graph->Labels()) {
         painter->setFont(QFont("times",10,QFont::Bold));
         painter->setPen(black);
         for(i = 0;i < N;i++) {
             if(graph->Label(i,str)) {
                painter->drawText(cx[i] - 40,cy[i] - 12,80,24,
                                  AlignHCenter|AlignVCenter|DontClip,str);
             }
         }
      }

      // Graphnummer ausgeben
      sprintf(str,"#%04d",num + 1);
      painter->setFont(QFont("times",10));
      painter->setPen(black);
      painter->drawText(x + 2,y,breite - 4,14,AlignLeft|AlignTop|DontClip,str);

      // Titel ausgeben
      if(graph->Titel()) {
         painter->setFont(QFont("times",8));
         painter->drawText(x,y + hoehe - 12,breite,12,
                           AlignHCenter|AlignBottom|DontClip,graph->Titel());
      }
   }
}


// Alle Graphen auf einer Seite zeichnen
void GraphWidget::ZeichneSeite(const card32 start,
                         QPainter* painter,
                         const card32 wx,
                         const card32 wy,
                         const card32 wbreite,
                         const card32 whoehe) const
{
   card32 breite = wbreite / anzahlX;
   card32 hoehe  = whoehe  / anzahlY;
   card32 num    = 0;
   Graph* graph = liste->at(start);

   // Graphen zeichnen
   for(card32 y = 0;y < (card32)anzahlY;y++) {
      for(card32 x = 0;x < (card32)anzahlX;x++) {
         ZeichneGraph(painter,
                      wx + x * breite,
                      wy + y * hoehe,breite,hoehe,
                      graph,start + num);
         num++;
         graph = liste->next();
         if(graph == NULL)
            return;
      }
   }
}


// Nchste Seite anzeigen
void GraphWidget::NaechsteSeite()
{
   int32 alt = position;
   position += anzahlX * anzahlY;
   if((card32)position >= liste->count()) {
      position = liste->count() - anzahlX * anzahlY;
      if(position < 0)
         position = 0;
   }
   if(position < alt) {
      position = alt;  
   }
   if(position != alt) {
      paintEvent(NULL);
      NeueSeite();
   }
}


// Vorherige Seite anzeigen
void GraphWidget::VorherigeSeite()
{
   int32 alt = position;
   position -= anzahlX * anzahlY;
   if(position < 0)
      position = 0; 
   if(position != alt) {
      paintEvent(NULL);
      NeueSeite();
   }
}


// Erste Seite anzeigen
void GraphWidget::ErsteSeite()
{
   if(position > 0) {
      position = 0;
      paintEvent(NULL);
      NeueSeite();
   }
}


// Letzte Seite anzeigen
void GraphWidget::LetzteSeite()
{
   int32 alt  = position;
   int32 rest = liste->count() % (anzahlX * anzahlY);
   if(rest > 0)
      position = liste->count() - rest;
    else
      position = liste->count() - (anzahlX * anzahlY);
   if(position < 0) position = 0;
   if(position != alt) {
      paintEvent(NULL);
      NeueSeite();
   }
}


// Neue Seitenzahl berechnen
void GraphWidget::NeueSeite()
{
   seite = (position / (anzahlX * anzahlY)) + 1;
   seiten = (int32)ceil((double)liste->count() / (double)(anzahlX * anzahlY));
   scrollBar->setRange(1,max(1,seiten));
   scrollBar->setValue(seite);
   neueSeite();   // Signal senden
}


// Einstellungen ndern
void GraphWidget::Einstellungen(int32 x, int32 y)
{
   anzahlX += x;
   anzahlY += y;
   if(anzahlX < 1) anzahlX = 1;
   if(anzahlY < 1) anzahlY = 1;
   position -= position % (anzahlX * anzahlY);
   if(position < 0) position = 0;
   NeueSeite();
   paintEvent(NULL);
}


// Neue Liste beginnen
void GraphWidget::Neu()
{
   Graph* graph = liste->first();
   while(graph != NULL) {
      liste->removeFirst();
      delete(graph);
      graph = liste->first();
   }
}


// Graph an Liste anfgen
void GraphWidget::FuegeAn(const Graph& g)
{
   Graph* graph = new Graph(g);
   liste->append(graph);
}


// Reset
void GraphWidget::Reset()
{
   position = 0;
   seite    = 1;
   paintEvent(NULL);
   NeueSeite();
}


// ScrollBar-Auswahl
void GraphWidget::scrollBarEvent(int seite)
{
   int32 alt = position;
   position = (seite - 1) * (anzahlX * anzahlY);
   if(position < 0) 
      position = 0;
   if(position >= (int32)liste->count()) {
      position = liste->count() - anzahlX * anzahlY;
      if(position < 0)
         position = 0;
   }
   if(position != alt) {
      paintEvent(NULL);
      NeueSeite();
   }
}

#include "graphwidget.moc"
