// ########################################################################
// ## zufall.cc                                                          ##
// ## Zufallsfunktionen                                                  ##
// ## Copyright Thomas Dreibholz, uzs6pq@ibm.rhrz.uni-bonn.de            ##
// ##           WWW Homepage:     http://ibm.rhrz.uni-bonn.de/~uzs6pq    ##
// ########################################################################

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cmath>

#include <iostream.h>
#include <fstream.h>

#include "../system.h"
#include "zufall.h"

 

// Zufallszahlengenerator mit Startwert von Systemzeit initialisieren
void Randomize()
{
   ifstream is(RANDOM_DEVICE,ios::in | ios::binary);
   if(is.good()) {
      card32 zahl;
      char   a,b,c,d;
      is >> a >> b >> c >> d;
      zahl = ((card32)a << 24) + ((card32)b << 16) + ((card32)c << 8) + d;
      srand(zahl);
   }
   else {
      time_t zeit;
      time(&zeit);
      srand(zeit);
   }
}

// 32-Bit-Zufallszahl im Bereich von 1 bis n erzeugen
card32 Random(card32 n)
{
   return( (random() % n) + (card32)1 );
}


// ##### Zufallszahlengeneration fr groe Zahlen #########################


// 64-Bit-Zufallszahl im Bereich von 1 bis n erzeugen
card64 Random(card64 n)
{
   card32 z1,z2;
   card64 z;
   z1 = random();
   z2 = random();
   z = (((card64)z1) << 32) + (card64)z2;
   return( (z % n) + (card64)1 );
}


// 32-Bit-Zufallszahl mit "bits" Bits erzeugen
// <= 0 Bits  -> Zahl = 0
// >= 32 Bits -> Zahl mit 32 Bits
card32 RandomI(int32 bits)
{
   card32 zahl = 0;
   if(bits > 32) bits=32;
   if(bits > 0) {
      for(int32 i = 0;i < bits;i++) {
         if(random() & 1) zahl |= (1 << i);
      }
   }
   return(zahl);
}


// Double-64-Bit-Zufallszahl im Bereich von 1 bis n erzeugen
double Random(double n)
{ 
   long double ergebnis;

   do {
      int32  laenge = 1 + (card32)(log((double)n) / log((double)2.0));
      card32 i,exp;
      card32 anzahl = (laenge + 31) / 32;

      ergebnis = 1.0;
      exp      = 0;
      for(i = 0;i < anzahl;i++) {
         ergebnis += (double)RandomI(laenge) * pow((double)2.0,(double)exp);
         laenge   -= 32;
         exp      += 32;
      }

   } while(ergebnis > n);

   return(ergebnis);
}


// Long Double-96-Bit-Zufallszahl im Bereich von 1 bis n erzeugen
long double Random(long double n)
{
   long double ergebnis;

   do {
      int32  laenge = 1 + (card32)(log((long double)n) / log((long double)2.0));
      card32 i,exp;
      card32 anzahl = (laenge + 31) / 32;

      // cout << "L=" <<  laenge << "  A=" << anzahl << "  =>  ";

      ergebnis = 1.0;
      exp      = 0;
      for(i = 0;i < anzahl;i++) {
         ergebnis += (long double)RandomI(laenge) * pow((long double)2.0,(long double)exp);
         laenge   -= 32;
         exp      += 32;
      }
  
      // cout << ergebnis << endl;

   } while(ergebnis > n);

   return(ergebnis);
}
